/******************************************************************************
* Copyright (C) 2021 Xilinx, Inc.  All rights reserved.
* SPDX-License-Identifier: MIT
******************************************************************************/
#ifndef BSPCONFIG_H
#define BSPCONFIG_H

#include "xmem_config.h"


#define EL3  0
#define EL1_NONSECURE  0
#define HYP_GUEST  0
#define versal  
/* #undef PSU_PMU */
/* #undef PLATFORM_ZYNQMP */
/* #undef PLATFORM_ZYNQ */
/* #undef VERSAL_PLM */
#define PLATFORM_MB  
#define SDT
#define XPAR_CPU_ID 0
/* #undef XPAR_STDIN_IS_UARTLITE */
/* #undef XPAR_STDIN_IS_UARTNS550 */
/* #undef XPAR_STDIN_IS_UARTPS */
#define XPAR_STDIN_IS_UARTPSV  
#define STDIN_BASEADDRESS 0xff000000
#define STDOUT_BASEADDRESS 0xff000000

#endif /* BSPCONFIG_H */
