#ifndef XMEM_CONFIG_H_
#define XMEM_CONFIG_H_

#define XPAR_AXI_NOC_0_BASEADDRESS 0x0
#define XPAR_AXI_NOC_0_HIGHADDRESS 0x80000000
#define XPAR_PSV_OCM_0_BASEADDRESS 0xfffc0000
#define XPAR_PSV_OCM_0_HIGHADDRESS 0x100000000

#endif